@@ -59,40 +59,173 @@ Note:
     getting into the weeds of how we coerce technically non-hashable sequences
     into a hashable encoding.
 """
+
+from __future__ import annotations
+
+import dataclasses
 import hashlib
 import math
+import typing
 from collections import OrderedDict
+
 from ubelt.util_const import NoParam
 
+if typing.TYPE_CHECKING:
+    from os import PathLike
+    from typing import Any, Callable, Optional, Union, cast
+
+    from ubelt.util_const import NoParamType
+    # A constructor / factory that returns a hashlib-style hash object.
+    # Kept behind TYPE_CHECKING to ensure near-zero runtime/import-time cost.
+
+    BytesLike = bytes | bytearray | memoryview
+
+    class HasherLike(typing.Protocol):
+        def update(self, data: BytesLike, /) -> None: ...
+        def digest(self) -> bytes: ...
+        def hexdigest(self) -> str: ...
+        def copy(
+            self,
+        ) -> 'HasherLike': ...  # or Self if you want (typing.Self on 3.11+)
+
+    HasherType = Callable[..., HasherLike]
+
+
 __all__ = ['hash_data', 'hash_file']
 
 # incremented when we make a change that modifies hashes
-HASH_VERSION = 2  # type: int
+HASH_VERSION: int = 2
 
-_ALPHABET_10 = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']  # type: List[str]
+_ALPHABET_10: list[str] = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']
 
-_ALPHABET_16 = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
-                'a', 'b', 'c', 'd', 'e', 'f']
+_ALPHABET_16: list[str] = [
+    '0',
+    '1',
+    '2',
+    '3',
+    '4',
+    '5',
+    '6',
+    '7',
+    '8',
+    '9',
+    'a',
+    'b',
+    'c',
+    'd',
+    'e',
+    'f',
+]
 
-_ALPHABET_26 = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
-                'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
-                'u', 'v', 'w', 'x', 'y', 'z']
+_ALPHABET_26: list[str] = [
+    'a',
+    'b',
+    'c',
+    'd',
+    'e',
+    'f',
+    'g',
+    'h',
+    'i',
+    'j',
+    'k',
+    'l',
+    'm',
+    'n',
+    'o',
+    'p',
+    'q',
+    'r',
+    's',
+    't',
+    'u',
+    'v',
+    'w',
+    'x',
+    'y',
+    'z',
+]
 
-_ALPHABET_36 = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
-                'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
-                'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
-                'u', 'v', 'w', 'x', 'y', 'z']
+_ALPHABET_36: list[str] = [
+    '0',
+    '1',
+    '2',
+    '3',
+    '4',
+    '5',
+    '6',
+    '7',
+    '8',
+    '9',
+    'a',
+    'b',
+    'c',
+    'd',
+    'e',
+    'f',
+    'g',
+    'h',
+    'i',
+    'j',
+    'k',
+    'l',
+    'm',
+    'n',
+    'o',
+    'p',
+    'q',
+    'r',
+    's',
+    't',
+    'u',
+    'v',
+    'w',
+    'x',
+    'y',
+    'z',
+]
 
 # RFC 4648 Base32 alphabet
-_ALPHABET_32 = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
-                'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
-                'Y', 'Z', '2', '3', '4', '5', '6', '7']
+_ALPHABET_32: list[str] = [
+    'A',
+    'B',
+    'C',
+    'D',
+    'E',
+    'F',
+    'G',
+    'H',
+    'I',
+    'J',
+    'K',
+    'L',
+    'M',
+    'N',
+    'O',
+    'P',
+    'Q',
+    'R',
+    'S',
+    'T',
+    'U',
+    'V',
+    'W',
+    'X',
+    'Y',
+    'Z',
+    '2',
+    '3',
+    '4',
+    '5',
+    '6',
+    '7',
+]
 
 
-DEFAULT_ALPHABET = _ALPHABET_16  # type: List[str]
+DEFAULT_ALPHABET: list[str] = _ALPHABET_16
 
 
-def b(s):
+def b(s: str) -> bytes:
     """
     Args:
         s (str):
@@ -100,7 +233,8 @@ def b(s):
     Returns:
         bytes
     """
-    return s.encode("latin-1")
+    return s.encode('latin-1')
+
 
 # Sensible choices for default hashers are sha1, sha512, and xxh64.
 
@@ -120,7 +254,7 @@ def b(s):
 # DEFAULT_HASHER = hashlib.sha1  # fast algo, but has a known collision
 # DEFAULT_HASHER = hashlib.sha512  # most robust algo, but slower than others
 
-DEFAULT_HASHER = hashlib.sha512  # type: Callable
+DEFAULT_HASHER: Callable[..., hashlib._Hash] = hashlib.sha512
 
 
 # This controls if types are used when generating hashable sequences for more
@@ -129,7 +263,7 @@ DEFAULT_HASHER = hashlib.sha512  # type: Callable
 _COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT = True
 
 
-# Note: the Hasher refers to hashlib._hashlib.HASH
+# Note: the HasherType refers to hashlib._hashlib.HASH
 # but this does not play well with type annotations
 # try:
 #     HASH = hashlib._hashlib.HASH
@@ -137,7 +271,8 @@ _COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT = True
 #     # Python seems to have been compiled without OpenSSL
 #     HASH = None
 
-def _int_to_bytes(int_):
+
+def _int_to_bytes(int_: int) -> bytes:
     r"""
     Converts an integer into its byte representation
     assumes int32 by default, but dynamically handles larger ints
@@ -164,7 +299,7 @@ def _int_to_bytes(int_):
     return bytes_
 
 
-def _bytes_to_int(bytes_):
+def _bytes_to_int(bytes_: bytes) -> int:
     r"""
     Converts a string of bytes into its integer representation (big-endian)
 
@@ -183,36 +318,37 @@ def _bytes_to_int(bytes_):
     return int_
 
 
-class _Hashers(object):
+class _Hashers:
     """
     We offer hashers beyond what is available in hashlib.
     This class is used to lazy load them.
 
     Attributes:
-        algos (Dict[str, object]):
-        aliases (Dict[str, str]):
+        algos (dict[str, object]):
+        aliases (dict[str, str]):
     """
-    def __init__(self):
-        self.algos = {}    # type: Dict[str, object]
-        self.aliases = {}  # type: Dict[str, str]
+
+    def __init__(self) -> None:
+        self.algos: dict[str, Callable] = {}
+        self.aliases: dict[str, str] = {}
         self._lazy_queue = [
             self._register_xxhash,
             self._register_blake3,
             self._register_hashlib,
         ]
 
-    def available(self):
+    def available(self) -> list[str]:
         """
         The names of available hash algorithms
 
         Returns:
-            List[str]
+            list[str]
         """
         if self._lazy_queue:  # nocover
             self._evaluate_registration_queue()
         return list(self.algos.keys())
 
-    def _evaluate_registration_queue(self):
+    def _evaluate_registration_queue(self) -> None:
         for func in self._lazy_queue:
             try:
                 func()
@@ -220,7 +356,7 @@ class _Hashers(object):
                 ...
         self._lazy_queue.clear()
 
-    def __contains__(self, key):
+    def __contains__(self, key: str) -> bool:
         """
         Args:
             key (str): name of hash algo to check
@@ -233,21 +369,26 @@ class _Hashers(object):
         return key in self.algos or key in self.aliases
 
     def _register_xxhash(self):  # nocover
-        import xxhash  # type: ignore
+        import xxhash
+
         self.algos['xxh32'] = xxhash.xxh32
         self.algos['xxh64'] = xxhash.xxh64
-        self.aliases.update({
-            'xxhash': 'xxh32',
-            'xx32': 'xxh32',
-            'xx64': 'xxh64',
-        })
+        self.aliases.update(
+            {
+                'xxhash': 'xxh32',
+                'xx32': 'xxh32',
+                'xx64': 'xxh64',
+            }
+        )
 
     def _register_blake3(self):  # nocover
-        import blake3  # type: ignore
+        import importlib
+
+        blake3 = importlib.import_module('blake3')
         self.algos['blake3'] = blake3.blake3
         self.aliases['b3'] = 'blake3'
 
-    def _register_hashlib(self):
+    def _register_hashlib(self) -> None:
         guaranteed = set(hashlib.algorithms_guaranteed)
         for key in guaranteed:  # nocover
             self.algos[key] = getattr(hashlib, key)
@@ -259,10 +400,10 @@ class _Hashers(object):
             for key in extra:
                 self.algos[key] = hashlib.new(key)
 
-    def lookup(self, hasher):
+    def lookup(self, hasher: NoParamType | str | Any) -> Callable[[], Any]:
         """
         Args:
-            hasher (NoParamType | str | Any):
+            hasher (NoParamType | str | object):
                 something coercible to a hasher
 
         Returns:
@@ -275,6 +416,8 @@ class _Hashers(object):
             # by default the result of this function is a class we will make an
             # instance of, if we already have an instance, wrap it in a
             # callable so the external syntax does not need to change.
+            if typing.TYPE_CHECKING:
+                hasher = cast(HasherLike, hasher)
             return lambda: hasher
         else:
             # Ensure lazy registration functions have been executed
@@ -283,14 +426,17 @@ class _Hashers(object):
 
             if isinstance(hasher, str):
                 hasher_ = self.aliases.get(hasher, hasher)
-                if hasher_ in self.algos:  # pragma: no cover
+                if hasher_ in self.algos:
                     return self.algos[hasher_]
                 else:
                     # TODO: provide pip install messages for known hashers.
                     raise KeyError('unknown hasher: {}'.format(hasher))
 
+        if typing.TYPE_CHECKING:
+            hasher = cast(HasherType, hasher)
         return hasher
 
+
 _HASHERS = _Hashers()
 
 
@@ -337,7 +483,7 @@ def _rectify_base(base):
     transforms base shorthand into the full list representation
 
     Args:
-        base (str | int | List[str]):
+        base (str | int | list[str] | tuple[str, ...]):
             Can be a list of characters in the base.
             Can be a number indicating the size of the base (only 26, 32, 26,
             16, and 10 are currently allowed).
@@ -374,11 +520,13 @@ def _rectify_base(base):
         if not isinstance(base, (list, tuple)):
             raise TypeError(
                 'Argument `base` must be a key, list, or tuple; not {}'.format(
-                    type(base)))
+                    type(base)
+                )
+            )
         return base
 
 
-class HashableExtensions(object):
+class HashableExtensions:
     """
     Helper class for managing non-primitive (e.g. numpy) hash types
 
@@ -387,25 +535,30 @@ class HashableExtensions(object):
         this class can be created and passed as arguments to hash_data.
 
     Attributes:
-        iterable_checks (List[Callable]):
+        iterable_checks (list[Callable[..., bool]]):
     """
-    def __init__(self):
-        self.iterable_checks = []
-        self._lazy_queue = []  # type: List[Callable]  # NOQA
+
+    def __init__(self) -> None:
+        self.iterable_checks: list[Callable[..., bool]] = []
+        self._lazy_queue: list[Callable[[], None]] = []
 
         # New singledispatch registry implementation
         from functools import singledispatch
+
         def _hash_dispatch(data):
             raise NotImplementedError
-        _hash_dispatch.__is_base__ = True
+
+        _hash_dispatch.__is_base__ = True  # type: ignore[unresolved-attribute]
         self._hash_dispatch = singledispatch(_hash_dispatch)
 
-    def _evaluate_lazy_queue(self):
+    def _evaluate_lazy_queue(self) -> None:
         for func in self._lazy_queue:
             func()
         self._lazy_queue.clear()
 
-    def register(self, hash_types):
+    def register(
+        self, hash_types: Union[type, tuple[type, ...], list[type]]
+    ) -> Callable[..., Any]:
         """
         Registers a function to generate a hash for data of the appropriate
         types. This can be used to register custom classes. Internally this is
@@ -416,7 +569,7 @@ class HashableExtensions(object):
         hashed.
 
         Args:
-            hash_types (type | Tuple[type]):
+            hash_types (type | tuple[type, ...] | list[type]):
 
         Returns:
             Callable: closure to be used as the decorator
@@ -424,7 +577,7 @@ class HashableExtensions(object):
         Example:
             >>> import ubelt as ub
             >>> import pytest
-            >>> class MyType(object):
+            >>> class MyType:
             ...     def __init__(self, id):
             ...         self.id = id
             >>> data = MyType(1)
@@ -451,9 +604,9 @@ class HashableExtensions(object):
             >>> import ubelt as ub
             >>> import pytest
             >>> extensions = ub.util_hash.HashableExtensions()
-            >>> class Type1(object):
+            >>> class Type1:
             >>>     ...
-            >>> class Type2(object):
+            >>> class Type2:
             >>>     ...
             >>> @extensions.register([Type1, Type2])
             >>> def hash_my_type(data):
@@ -469,7 +622,7 @@ class HashableExtensions(object):
             >>> # the global state.
             >>> import ubelt as ub
             >>> import pytest
-            >>> class MyType(object):
+            >>> class MyType:
             ...     def __init__(self, id):
             ...         self.id = id
             >>> data = MyType(1)
@@ -495,13 +648,15 @@ class HashableExtensions(object):
         # ensure iterable
         if not isinstance(hash_types, (list, tuple)):
             hash_types = [hash_types]
+
         def _decor_closure(hash_func):
             for hash_type in hash_types:
                 self._hash_dispatch.register(hash_type)(hash_func)
             return hash_func
+
         return _decor_closure
 
-    def lookup(self, data):
+    def lookup(self, data: object) -> Callable[..., tuple[bytes, bytes]]:
         """
         Returns an appropriate function to hash ``data`` if one has been
         registered.
@@ -528,7 +683,7 @@ class HashableExtensions(object):
             >>> data = np.array([1, 2, 3])
             >>> self.lookup(data[0])
 
-            >>> class Foo(object):
+            >>> class Foo:
             >>>     def __init__(f):
             >>>         f.attr = 1
             >>> data = Foo()
@@ -567,12 +722,18 @@ class HashableExtensions(object):
             # is unclear how to build a test in for this.
             self._evaluate_lazy_queue()
 
+        if dataclasses.is_dataclass(data):
+            # Handle dataclasses out of the box
+            return self._hash_dataclass
+
         query_hash_type = data.__class__
         # TODO: recognize some special dunder method instead
         # of strictly using this registry.
         hash_func = self._hash_dispatch.dispatch(query_hash_type)
         if getattr(hash_func, '__is_base__', False):
-            base_msg = f'No registered hash func for hashable type={query_hash_type!r}'
+            base_msg = (
+                f'No registered hash func for hashable type={query_hash_type!r}'
+            )
             try:
                 msg = f'{base_msg} with mro: {query_hash_type.__mro__}'
             except AttributeError:  # nocover
@@ -580,7 +741,47 @@ class HashableExtensions(object):
             raise TypeError(msg)
         return hash_func
 
-    def add_iterable_check(self, func):
+    def _hash_dataclass(self, data):
+        """
+        Dataclasses don't dispatch.
+
+        Example:
+            >>> from dataclasses import dataclass
+            >>> import ubelt as ub
+            >>> #
+            >>> @dataclass
+            >>> class P:
+            >>>     x: int
+            >>>     y: int
+            >>> #
+            >>> a = P(1, 2)
+            >>> b = P(1, 2)
+            >>> c = P(2, 1)
+            >>> #
+            >>> assert ub.hash_data(a) == ub.hash_data(b)
+            >>> assert ub.hash_data(a) != ub.hash_data(c)
+        """
+        import dataclasses
+
+        cls = data.__class__
+        header = (cls.__module__, cls.__qualname__)
+        # fields() order is the definition order, which is guaranteed
+        items = [
+            (f.name, getattr(data, f.name)) for f in dataclasses.fields(data)
+        ]
+        # Use the existing machinery to serialize recursively
+        seq = _hashable_sequence(
+            (header, items),
+            extensions=self,
+            types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+        )
+        prefix = b'DCLASS'
+        hashable = b''.join(seq)
+        return prefix, hashable
+
+    def add_iterable_check(
+        self, func: Callable[..., bool]
+    ) -> Callable[..., bool]:
         """
         Registers a function that detects when a type is iterable
 
@@ -631,12 +832,20 @@ class HashableExtensions(object):
                 # tobytes() views the array in 1D (via ravel())
                 # encode the shape as well
                 # See: [util_hash.Note.1]
-                header = b''.join(_hashable_sequence(
-                    (len(data.shape), data.shape), extensions=self,
-                    types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT))
-                dtype = b''.join(_hashable_sequence(
-                    data.dtype.descr, extensions=self,
-                    types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT))
+                header = b''.join(
+                    _hashable_sequence(
+                        (len(data.shape), data.shape),
+                        extensions=self,
+                        types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+                    )
+                )
+                dtype = b''.join(
+                    _hashable_sequence(
+                        data.dtype.descr,
+                        extensions=self,
+                        types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+                    )
+                )
                 hashable = header + dtype + data.tobytes()
             prefix = b'NDARR'
             return prefix, hashable
@@ -653,9 +862,13 @@ class HashableExtensions(object):
                 >>> _hashable_sequence(rng, types=True)
             """
             # See: [util_hash.Note.1]
-            hashable = b''.join(_hashable_sequence(
-                data.get_state(), extensions=self,
-                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT))
+            hashable = b''.join(
+                _hashable_sequence(
+                    data.get_state(),
+                    extensions=self,
+                    types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+                )
+            )
             prefix = b'RNG'
             return prefix, hashable
 
@@ -746,11 +959,11 @@ class HashableExtensions(object):
         # TODO: can we only register a stdlib class if we need it?
         # Some of the stdlib modules dont need to be imported and
         # cause extra import time overhead.
-        import uuid
-        import pathlib
-        import numbers
-        import decimal
         import datetime as datetime_mod
+        import decimal
+        import numbers
+        import pathlib
+        import uuid
 
         @self.register(numbers.Integral)
         def _convert_numpy_int(data):
@@ -766,7 +979,8 @@ class HashableExtensions(object):
             seq = _hashable_sequence(
                 data.as_tuple(),
                 extensions=self,
-                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT)
+                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+            )
             hashable = b''.join(seq)
             prefix = b'DECIMAL'
             return prefix, hashable
@@ -777,7 +991,8 @@ class HashableExtensions(object):
             seq = _hashable_sequence(
                 data.timetuple(),
                 extensions=self,
-                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT)
+                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+            )
             hashable = b''.join(seq)
             prefix = b'DATE'
             return prefix, hashable
@@ -788,7 +1003,8 @@ class HashableExtensions(object):
             seq = _hashable_sequence(
                 data.timetuple(),
                 extensions=self,
-                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT)
+                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+            )
             hashable = b''.join(seq)
             prefix = b'DATETIME'
             return prefix, hashable
@@ -806,13 +1022,18 @@ class HashableExtensions(object):
                 ordered_ = sorted(data)
             except TypeError:
                 from ubelt.util_list import argsort
+
                 data_ = list(data)
                 sortx = argsort(data_, key=str)
                 ordered_ = [data_[k] for k in sortx]
             # See: [util_hash.Note.1]
-            hashable = b''.join(_hashable_sequence(
-                ordered_, extensions=self,
-                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT))
+            hashable = b''.join(
+                _hashable_sequence(
+                    ordered_,
+                    extensions=self,
+                    types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+                )
+            )
             prefix = b'SET'
             return prefix, hashable
 
@@ -823,12 +1044,17 @@ class HashableExtensions(object):
                 # what raises a TypeError differs between Python 2 and 3
             except TypeError:
                 from ubelt.util_list import argsort
+
                 sortx = argsort(data, key=str)
                 ordered_ = [(k, data[k]) for k in sortx]
             # See: [util_hash.Note.1]
-            hashable = b''.join(_hashable_sequence(
-                ordered_, extensions=self,
-                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT))
+            hashable = b''.join(
+                _hashable_sequence(
+                    ordered_,
+                    extensions=self,
+                    types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+                )
+            )
             prefix = b'DICT'
             return prefix, hashable
 
@@ -839,9 +1065,13 @@ class HashableExtensions(object):
             regular dictionaries. I'm not sure what the right thing to do is.
             """
             # See: [util_hash.Note.1]
-            hashable = b''.join(_hashable_sequence(
-                list(data.items()), extensions=self,
-                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT))
+            hashable = b''.join(
+                _hashable_sequence(
+                    list(data.items()),
+                    extensions=self,
+                    types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+                )
+            )
             prefix = b'ODICT'
             return prefix, hashable
 
@@ -852,9 +1082,13 @@ class HashableExtensions(object):
             regular dictionaries. I'm not sure what the right thing to do is.
             """
             # See: [util_hash.Note.1]
-            hashable = b''.join(_hashable_sequence(
-                [data.start, data.stop, data.step], extensions=self,
-                types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT))
+            hashable = b''.join(
+                _hashable_sequence(
+                    [data.start, data.stop, data.step],
+                    extensions=self,
+                    types=_COMPATIBLE_HASHABLE_SEQUENCE_TYPES_DEFAULT,
+                )
+            )
             prefix = b'SLICE'
             return prefix, hashable
 
@@ -875,13 +1109,17 @@ class HashableExtensions(object):
         Experimental. Define a default hash function for torch tensors, but do
         not use it by default. Currently, the user must call this explicitly.
         """
-        import torch
+        import importlib
+
+        torch = importlib.import_module('torch')
+
         @self.register(torch.Tensor)
         def _convert_torch_tensor(data):
             data_ = data.data.cpu().numpy()
             prefix = b'TORCH_TENSOR'
             return prefix, _convert_to_hashable(data_, extensions=self)[1]
 
+
 _HASHABLE_EXTENSIONS = HashableExtensions()
 
 
@@ -900,25 +1138,25 @@ def _lazy_init():
 _HASHABLE_EXTENSIONS._lazy_queue.append(_lazy_init)
 
 
-class _HashTracer(object):
+class _HashTracer:
     """
     Helper class to extract hashed sequences
 
     Attributes:
-        sequence (List[bytes]):
+        sequence (list[bytes]):
     """
 
-    def __init__(self):
-        self.sequence = []  # type: List[bytes]
+    def __init__(self) -> None:
+        self.sequence: list[bytes] = []
 
-    def update(self, item):
+    def update(self, item: bytes) -> None:
         """
         Args:
             item (bytes):
         """
         self.sequence.append(item)
 
-    def hexdigest(self):
+    def hexdigest(self) -> bytes:
         """
         Returns:
             bytes
@@ -952,7 +1190,7 @@ def _convert_to_hashable(data, types=True, extensions=None):
         types (bool): include type prefixes in the hash
 
     Returns:
-        Tuple[bytes, bytes]: prefix, hashable:
+        tuple[bytes, bytes]: prefix, hashable:
             a prefix hinting the original data type and the byte representation
             of ``data``.
 
@@ -999,7 +1237,7 @@ def _convert_to_hashable(data, types=True, extensions=None):
         except (ValueError, OverflowError):
             hashable = str(data_).encode('utf-8')  # handle and nan, inf
         else:
-            hashable = _int_to_bytes(a) + b'/' +  _int_to_bytes(b)
+            hashable = _int_to_bytes(a) + b'/' + _int_to_bytes(b)
         prefix = b'FLT'
     else:
         if extensions is None:
@@ -1027,7 +1265,7 @@ def _update_hasher(hasher, data, types=True, extensions=None):
     :class:`hashlib._hashlib.HASH` algorithm.
 
     Args:
-        hasher (Hasher): instance of a hashlib algorithm
+        hasher (HasherType): instance of a hashlib algorithm
         data (object): ordered data with structure
         types (bool): include type prefixes in the hash
         extensions (HashableExtensions | None): overrides global extensions
@@ -1051,8 +1289,9 @@ def _update_hasher(hasher, data, types=True, extensions=None):
     if isinstance(data, (tuple, list, zip)):
         needs_iteration = True
     else:
-        needs_iteration = any(check(data) for check in
-                              extensions.iterable_checks)
+        needs_iteration = any(
+            check(data) for check in extensions.iterable_checks
+        )
 
     if needs_iteration:
         # Denote that we are hashing over an iterable
@@ -1068,8 +1307,9 @@ def _update_hasher(hasher, data, types=True, extensions=None):
         # (this works if all data in the sequence is a non-iterable)
         try:
             for item in iter_:
-                prefix, hashable = _convert_to_hashable(item, types,
-                                                        extensions=extensions)
+                prefix, hashable = _convert_to_hashable(
+                    item, types, extensions=extensions
+                )
                 binary_data = prefix + hashable + _SEP
                 hasher.update(binary_data)
             hasher.update(_ITER_SUFFIX)
@@ -1089,8 +1329,9 @@ def _update_hasher(hasher, data, types=True, extensions=None):
                 hasher.update(_SEP)
             hasher.update(_ITER_SUFFIX)
     else:
-        prefix, hashable = _convert_to_hashable(data, types,
-                                                extensions=extensions)
+        prefix, hashable = _convert_to_hashable(
+            data, types, extensions=extensions
+        )
         binary_data = prefix + hashable
         hasher.update(binary_data)
 
@@ -1164,7 +1405,7 @@ def _convert_hexstr_base(hexstr, base):
 
 
 def _digest_hasher(hasher, base):
-    """ counterpart to _update_hasher """
+    """counterpart to _update_hasher"""
     # Get a 128 character hex string
     hex_text = hasher.hexdigest()
     # Shorten length of string (by increasing base)
@@ -1175,8 +1416,14 @@ def _digest_hasher(hasher, base):
 
 
 # @profile
-def hash_data(data, hasher=NoParam, base=NoParam, types=False, convert=False,
-              extensions=None):
+def hash_data(
+    data: object,
+    hasher: str | HasherType | NoParamType = NoParam,
+    base: Union[list[str], tuple[str, ...], str, NoParamType] = NoParam,
+    types: bool = False,
+    convert: bool = False,
+    extensions: HashableExtensions | None = None,
+) -> str:
     """
     Get a unique hash depending on the state of the data.
 
@@ -1184,13 +1431,13 @@ def hash_data(data, hasher=NoParam, base=NoParam, types=False, convert=False,
         data (object):
             Any sort of loosely organized data
 
-        hasher (str | Hasher | NoParamType):
+        hasher (str | HasherType | NoParamType):
             string code or a hash algorithm from hashlib. Valid hashing
             algorithms are defined by :py:obj:`hashlib.algorithms_guaranteed`
             (e.g.  'sha1', 'sha512', 'md5') as well as 'xxh32' and 'xxh64' if
             :mod:`xxhash` is installed. Defaults to 'sha512'.
 
-        base (List[str] | str | NoParamType):
+        base (list[str] | tuple[str, ...] | str | NoParamType):
             list of symbols or shorthand key.
             Valid keys are 'dec', 'hex', 'abc', and 'alphanum', 10, 16, 26, 32.
             Defaults to 'hex'.
@@ -1234,6 +1481,7 @@ def hash_data(data, hasher=NoParam, base=NoParam, types=False, convert=False,
     """
     if convert and not isinstance(data, str):  # nocover
         import json
+
         try:
             data = json.dumps(data)
         except TypeError:
@@ -1250,8 +1498,14 @@ def hash_data(data, hasher=NoParam, base=NoParam, types=False, convert=False,
     return text
 
 
-def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None,
-              hasher=NoParam, base=NoParam):
+def hash_file(
+    fpath: Union[str, PathLike[str]],
+    blocksize: int = 1048576,
+    stride: int = 1,
+    maxbytes: Optional[int] = None,
+    hasher: Union[str, HasherType, NoParamType] = NoParam,
+    base: Union[list[str], tuple[str, ...], int, str, NoParamType] = NoParam,
+) -> str:
     r"""
     Hashes the data in a file on disk.
 
@@ -1259,7 +1513,7 @@ def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None,
     sha1sum, sha512sum, md5sum, etc...)
 
     Args:
-        fpath (PathLike):
+        fpath (str | PathLike[str]):
             location of the file to be hashed.
 
         blocksize (int):
@@ -1276,13 +1530,13 @@ def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None,
         maxbytes (int | None):
             if specified, only hash the leading `maxbytes` of data in the file.
 
-        hasher (str | Hasher | NoParamType):
+        hasher (str | HasherType | NoParamType):
             string code or a hash algorithm from hashlib. Valid hashing
             algorithms are defined by :py:obj:`hashlib.algorithms_guaranteed`
             (e.g.  'sha1', 'sha512', 'md5') as well as 'xxh32' and 'xxh64' if
             :mod:`xxhash` is installed.  Defaults to 'sha512'.
 
-        base (List[str] | int | str | NoParamType):
+        base (list[str] | tuple[str, ...] | int | str | NoParamType):
             list of symbols or shorthand key.
             Valid keys are 'dec', 'hex', 'abc', and 'alphanum', 10, 16, 26, 32.
             Defaults to 'hex'.
@@ -1414,8 +1668,8 @@ def hash_file(fpath, blocksize=1048576, stride=1, maxbytes=None,
 
 # Give the hash_data function itself a reference to the default extensions
 # register method so the user can modify them without accessing this module
-hash_data.extensions = _HASHABLE_EXTENSIONS
-hash_data.register = _HASHABLE_EXTENSIONS.register
+hash_data.extensions = _HASHABLE_EXTENSIONS  # type: ignore[attr-defined]
+hash_data.register = _HASHABLE_EXTENSIONS.register  # type: ignore[attr-defined]
 
 
 # class Hasher:
