@@ -2,7 +2,7 @@ r"""
 Expose functions to simplify importing from module names and paths.
 
 The :func:`ubelt.import_module_from_path` function does its best to load a
-python file into th current set of global modules.
+python file into the current set of global modules.
 
 The :func:`ubelt.import_module_from_name` works similarly.
 
@@ -12,11 +12,31 @@ statically and convert between module names and file paths on disk.
 The :func:`ubelt.split_modpath` function separates modules into a root and base
 path depending on where the first ``__init__.py`` file is.
 """
-from os.path import (abspath, basename, dirname, exists, expanduser, isdir,
-                     isfile, join, realpath, relpath, split, splitext)
+
+from __future__ import annotations
+
 import os
 import sys
+import typing
 import warnings
+from os.path import (
+    abspath,
+    basename,
+    dirname,
+    exists,
+    expanduser,
+    isdir,
+    isfile,
+    join,
+    realpath,
+    relpath,
+    split,
+    splitext,
+)
+
+if typing.TYPE_CHECKING:
+    from types import ModuleType, TracebackType
+    from typing import Type
 
 __all__ = [
     'split_modpath',
@@ -26,11 +46,11 @@ __all__ = [
     'import_module_from_path',
 ]
 
-IS_PY_GE_308 = sys.version_info[0:2] >= (3, 8)  # type: bool
-IS_PY_LT_314 = sys.version_info[0:2] < (3, 14)  # type: bool
+IS_PY_GE_308: bool = sys.version_info[0:2] >= (3, 8)
+IS_PY_LT_314: bool = sys.version_info[0:2] < (3, 14)
 
 
-class PythonPathContext(object):
+class PythonPathContext:
     """
     Context for temporarily adding a dir to the PYTHONPATH.
 
@@ -80,7 +100,11 @@ class PythonPathContext(object):
         >>> with pytest.raises(RuntimeError):
         >>>     self.__exit__(None, None, None)
     """
-    def __init__(self, dpath, index=0):
+
+    dpath: str
+    index: int
+
+    def __init__(self, dpath: str | os.PathLike, index: int = 0) -> None:
         """
         Args:
             dpath (str | PathLike): directory to insert into the PYTHONPATH
@@ -89,12 +113,17 @@ class PythonPathContext(object):
         self.dpath = os.fspath(dpath)
         self.index = index
 
-    def __enter__(self):
+    def __enter__(self) -> None:
         if self.index < 0:
             self.index = len(sys.path) + self.index + 1
         sys.path.insert(self.index, self.dpath)
 
-    def __exit__(self, ex_type, ex_value, ex_traceback):
+    def __exit__(
+        self,
+        ex_type: Type[BaseException] | None,
+        ex_value: BaseException | None,
+        ex_traceback: TracebackType | None,
+    ) -> bool | None:
         """
         Args:
             ex_type (Type[BaseException] | None):
@@ -105,11 +134,13 @@ class PythonPathContext(object):
             bool | None
         """
         need_recover = False
+        msg_parts: list[str] = []
         if len(sys.path) <= self.index:  # nocover
             msg_parts = [
                 'sys.path changed while in PythonPathContext.',
                 'len(sys.path) = {!r} but index is {!r}'.format(
-                    len(sys.path), self.index),
+                    len(sys.path), self.index
+                ),
             ]
             need_recover = True
 
@@ -120,7 +151,7 @@ class PythonPathContext(object):
                 'Expected dpath={!r} at index={!r} in sys.path, but got '
                 'dpath={!r}'.format(
                     self.dpath, self.index, sys.path[self.index]
-                )
+                ),
             ]
             need_recover = True
 
@@ -134,17 +165,22 @@ class PythonPathContext(object):
             else:
                 # We were able to recover, but warn the user. This method of
                 # recovery is a heuristic and does not work in some cases.
-                msg_parts.append((
-                    'Expected dpath was at index {}. '
-                    'This could indicate conflicting module namespaces.'
-                ).format(real_index))
+                msg_parts.append(
+                    (
+                        'Expected dpath was at index {}. '
+                        'This could indicate conflicting module namespaces.'
+                    ).format(real_index)
+                )
                 warnings.warn('\n'.join(msg_parts))
                 sys.path.pop(real_index)
         else:
             sys.path.pop(self.index)
 
 
-def import_module_from_path(modpath, index=-1):
+def import_module_from_path(
+    modpath: str | os.PathLike,
+    index: int = -1,
+) -> ModuleType:
     """
     Imports a module via a filesystem path.
 
@@ -274,9 +310,47 @@ def import_module_from_path(modpath, index=-1):
                 zimp_file = zipimport.zipimporter(archivepath)
                 try:
                     try:
-                        module = zimp_file.load_module(modname)
+                        # module = zimp_file.load_module(modname)
+                        if hasattr(zimp_file, 'exec_module'):
+                            import importlib
+
+                            # Modern path (3.4+; preferred, no deprecation)
+                            spec = importlib.util.spec_from_loader(
+                                modname, zimp_file
+                            )
+                            if spec is None:
+                                raise ImportError(
+                                    f'Cannot create spec for {modname!r}'
+                                )
+                            module = importlib.util.module_from_spec(spec)
+                            sys.modules[modname] = (
+                                module  # important for recursive imports
+                            )
+                            zimp_file.exec_module(module)
+                        else:  # nocover
+                            # Legacy fallback (deprecated; only used on very old Pythons)
+                            module = zimp_file.load_module(modname)
                     except Exception:  # nocover
-                        module = zimp_file.load_module(modname.replace('\\', '/'))  # hack
+                        _modname = modname.replace('\\', '/')  # hack
+                        if hasattr(zimp_file, 'exec_module'):
+                            import importlib
+
+                            # Modern path (3.4+; preferred, no deprecation)
+                            spec = importlib.util.spec_from_loader(
+                                _modname, zimp_file
+                            )
+                            if spec is None:
+                                raise ImportError(
+                                    f'Cannot create spec for {modname!r}'
+                                )
+                            module = importlib.util.module_from_spec(spec)
+                            sys.modules[_modname] = (
+                                module  # important for recursive imports
+                            )
+                            zimp_file.exec_module(module)
+                        else:
+                            # Legacy fallback (deprecated; only used on very old Pythons)
+                            module = zimp_file.load_module(_modname)
                 except Exception as ex:  # nocover
                     text = (
                         'Encountered error in import_module_from_path '
@@ -291,7 +365,8 @@ def import_module_from_path(modpath, index=-1):
                         internal=internal,
                         modname=modname,
                         archivepath=archivepath,
-                        ex=ex)
+                        ex=ex,
+                    )
                     raise Exception(text)
 
                 return module
@@ -304,7 +379,7 @@ def import_module_from_path(modpath, index=-1):
         return module
 
 
-def import_module_from_name(modname):
+def import_module_from_name(modname: str) -> ModuleType:
     """
     Imports a module from its string name (i.e. ``__name__``)
 
@@ -335,21 +410,10 @@ def import_module_from_name(modname):
         >>> assert [m.__name__ for m in modules] == modname_list
         >>> assert all(m in sys.modules for m in modname_list)
     """
-    if True:
-        # See if this fixes the Docker issue we saw but were unable to
-        # reproduce on another environment. Either way its better to use the
-        # standard importlib implementation than the one I wrote a long time
-        # ago.
-        import importlib
-        module = importlib.import_module(modname)
-    else:  # nocover
-        # The __import__ statement is weird
-        if '.' in modname:
-            fromlist = modname.split('.')[-1]
-            fromlist_ = list(map(str, fromlist))  # needs to be ascii for python2.7
-            module = __import__(modname, {}, {}, fromlist_, 0)
-        else:
-            module = __import__(modname, {}, {}, [], 0)
+    # Modern python has a stdlib solution for this.
+    import importlib
+
+    module = importlib.import_module(modname)
     return module
 
 
@@ -361,6 +425,7 @@ def _extension_module_tags():
         List[str]
     """
     import sysconfig
+
     tags = []
     # handle PEP 3149 -- ABI version tagged .so files
     # ABI = application binary interface
@@ -381,6 +446,7 @@ def _platform_pylib_exts():  # nocover
         tuple
     """
     import sysconfig
+
     valid_exts = []
     # return with and without API flags
     # handle PEP 3149 -- ABI version tagged .so files
@@ -391,7 +457,9 @@ def _platform_pylib_exts():  # nocover
     return tuple(valid_exts)
 
 
-def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
+def _syspath_modname_to_modpath(
+    modname, sys_path=None, exclude=None
+) -> str | None:
     """
     syspath version of modname_to_modpath
 
@@ -407,7 +475,7 @@ def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
             Defaults to None.
 
     Returns:
-        str: path to the module.
+        str | None: path to the module or None if it does not exist.
 
     Note:
         This is much slower than the pkgutil mechanisms.
@@ -426,12 +494,11 @@ def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
         ...static_analysis.py
         >>> print(_syspath_modname_to_modpath('xdoctest'))
         ...xdoctest
-        >>> # xdoctest: +REQUIRES(CPython)
-        >>> print(_syspath_modname_to_modpath('_ctypes'))
-        ..._ctypes...
+        >>> print(_syspath_modname_to_modpath('json'))
+        ...json
         >>> assert _syspath_modname_to_modpath('xdoctest', sys_path=[]) is None
         >>> assert _syspath_modname_to_modpath('xdoctest.static_analysis', sys_path=[]) is None
-        >>> assert _syspath_modname_to_modpath('_ctypes', sys_path=[]) is None
+        >>> assert _syspath_modname_to_modpath('json', sys_path=[]) is None
         >>> assert _syspath_modname_to_modpath('this', sys_path=[]) is None
 
     Example:
@@ -484,15 +551,18 @@ def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
     candidate_dpaths = ['.' if p == '' else p for p in sys_path]
 
     if exclude:
+
         def normalize(p):
             if sys.platform.startswith('win32'):  # nocover
                 return realpath(p).lower()
             else:
                 return realpath(p)
+
         # Keep only the paths not in exclude
         real_exclude = {normalize(p) for p in exclude}
-        candidate_dpaths = [p for p in candidate_dpaths
-                            if normalize(p) not in real_exclude]
+        candidate_dpaths = [
+            p for p in candidate_dpaths if normalize(p) not in real_exclude
+        ]
 
     def check_dpath(dpath):
         # Check for directory-based modules (has precedence over files)
@@ -530,7 +600,7 @@ def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
     found_modpath = None
     for dpath in candidate_dpaths:
         modpath = check_dpath(dpath)
-        if modpath:
+        if modpath:  # pragma: nobranch
             found_modpath = modpath
             break
 
@@ -542,7 +612,9 @@ def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
         # Basically a finder will be used for "complex" structures and
         # basic pth will be used for "simple" structures (which means has a
         # src/modname folder).
-        new_editable_finder_paths = sorted(glob.glob(join(dpath, _editable_fname_finder_py_pat)))
+        new_editable_finder_paths = sorted(
+            glob.glob(join(dpath, _editable_fname_finder_py_pat))
+        )
         if new_editable_finder_paths:  # nocover
             # This makes some assumptions, which may not hold in general
             # We may need to fallback entirely on pkgutil, which would
@@ -560,7 +632,9 @@ def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
                     except KeyError:
                         ...
                     else:
-                        if not exclude or normalize(target) not in real_exclude:  # pragma: nobranch
+                        if (
+                            not exclude or normalize(target) not in real_exclude
+                        ):  # pragma: nobranch
                             modpath = check_dpath(target)
                             if modpath:  # pragma: nobranch
                                 found_modpath = modpath
@@ -570,15 +644,20 @@ def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
 
         # If a finder does not exist, then the __editable__ pth file might hold
         # the path itself. Check for that.
-        new_editable_pth_paths = sorted(glob.glob(join(dpath, _editable_fname_pth_pat)))
+        new_editable_pth_paths = sorted(
+            glob.glob(join(dpath, _editable_fname_pth_pat))
+        )
         if new_editable_pth_paths:  # nocover
             # Disable coverage because the test that covers this is too slow.
             # It can be made faster, re-enable when that lands.
             import pathlib
+
             for editable_pth in new_editable_pth_paths:
                 editable_pth = pathlib.Path(editable_pth)
                 target = editable_pth.read_text().strip().split('\n')[-1]
-                if not exclude or normalize(target) not in real_exclude:
+                if (
+                    not exclude or normalize(target) not in real_exclude
+                ):  # pragma: nobranch
                     modpath = check_dpath(target)
                     if modpath:  # pragma: nobranch
                         found_modpath = modpath
@@ -608,12 +687,16 @@ def _syspath_modname_to_modpath(modname, sys_path=None, exclude=None):
             # The docs state there should only be one line, but I see two.
             with open(linkpath, 'r') as file:
                 target = file.readline().strip()
-            if not exclude or normalize(target) not in real_exclude:
+            if (
+                not exclude or normalize(target) not in real_exclude
+            ):  # pragma: nobranch
                 modpath = check_dpath(target)
-                if modpath:
+                if modpath:  # pragma: nobranch
                     found_modpath = modpath
                     break
 
+    if typing.TYPE_CHECKING:
+        found_modpath = typing.cast(str | None, found_modpath)
     return found_modpath
 
 
@@ -624,10 +707,11 @@ def _custom_import_modpath(modpath, index=-1):
         with PythonPathContext(dpath, index=index):
             module = import_module_from_name(modname)
     except Exception as ex:  # nocover
-        msg_parts = [(
-            'ERROR: Failed to import modname={} with modpath={} and '
-            'sys.path modified with {} at index={}').format(
-                modname, modpath, repr(dpath), index)
+        msg_parts = [
+            (
+                'ERROR: Failed to import modname={} with modpath={} and '
+                'sys.path modified with {} at index={}'
+            ).format(modname, modpath, repr(dpath), index)
         ]
         msg_parts.append('Caused by: {}'.format(repr(ex)))
         raise RuntimeError('\n'.join(msg_parts))
@@ -644,52 +728,67 @@ def _importlib_import_modpath(modpath):  # nocover
     dpath, rel_modpath = split_modpath(modpath)
     modname = modpath_to_modname(modpath)
     import importlib.util
+
     spec = importlib.util.spec_from_file_location(modname, modpath)
+    assert spec is not None
     module = importlib.util.module_from_spec(spec)
+    assert spec.loader is not None
     spec.loader.exec_module(module)
     return module
 
 
 def _importlib_modname_to_modpath(modname):  # nocover
-    import importlib.util
-    spec = importlib.util.find_spec(modname)
-    modpath = spec.origin.replace('.pyc', '.py')
-    return modpath
-
-
-def _pkgutil_modname_to_modpath(modname):  # nocover
     """
-    faster version of :func:`_syspath_modname_to_modpath` using builtin python
-    mechanisms, but unfortunately it doesn't play nice with pytest.
-
-    Note:
-        pkgutil.find_loader is deprecated in 3.12 and removed in 3.14
+    faster version of :func:`_syspath_modname_to_modpath` using builtin
+    python mechanisms, but unfortunately it doesn't play nice with pytest.
 
     Args:
         modname (str): the module name.
 
+    Note:
+        This has a side effect when you import subpckages, it forces the parent
+        package itself to import!
+
     Example:
         >>> # xdoctest: +SKIP
-        >>> from ubelt.util_import import _pkgutil_modname_to_modpath
+        >>> from ubelt.util_import import _importlib_modname_to_modpath
         >>> modname = 'xdoctest.static_analysis'
-        >>> _pkgutil_modname_to_modpath(modname)
+        >>> _importlib_modname_to_modpath(modname)
         ...static_analysis.py
-        >>> # xdoctest: +REQUIRES(CPython)
-        >>> _pkgutil_modname_to_modpath('_ctypes')
-        ..._ctypes...
+        >>> _importlib_modname_to_modpath('json')
+        ...json...
 
     Ignore:
-        >>> _pkgutil_modname_to_modpath('cv2')
+        >>> _importlib_modname_to_modpath('cv2')
+        >>> from ubelt.util_import import _importlib_modname_to_modpath
+        >>> from ubelt.util_import import _syspath_modname_to_modpath
+        >>> import timerit
+        >>> for _ in timerit(label='ours'):
+        >>>     _syspath_modname_to_modpath('xdoctest.static_analysis')
+        >>> for _ in timerit(label='importlib'):
+        >>>     _importlib_modname_to_modpath('xdoctest.static_analysis')
+        Timed ours for: 10 loops, best of 5
+            time per loop: best=20.237 ms, mean=20.244 ± 0.0 ms
+        Timed stdlib for: 445407 loops, best of 5
+            time per loop: best=387.000 ns, mean=424.680 ± 19.7 ns
     """
-    import pkgutil
-    loader = pkgutil.find_loader(modname)
-    if loader is None:
-        raise Exception('No module named {} in the PYTHONPATH'.format(modname))
-    modpath = loader.get_filename().replace('.pyc', '.py')
+    import importlib.util
+
+    spec = importlib.util.find_spec(modname)
+    assert spec is not None
+    assert spec.origin is not None
+    modpath = spec.origin.replace(
+        '.pyc', '.py'
+    )  # is pyc replace needed anymore?
     return modpath
 
 
-def modname_to_modpath(modname, hide_init=True, hide_main=False, sys_path=None):
+def modname_to_modpath(
+    modname: str,
+    hide_init: bool = True,
+    hide_main: bool = False,
+    sys_path: list[str | os.PathLike] | None = None,
+) -> str | None:
     """
     Finds the path to a python module from its name.
 
@@ -727,9 +826,8 @@ def modname_to_modpath(modname, hide_init=True, hide_main=False, sys_path=None):
         >>> modname = 'xdoctest'
         >>> modpath = modname_to_modpath(modname, hide_init=False)
         >>> assert modpath.endswith('__init__.py')
-        >>> # xdoctest: +REQUIRES(CPython)
-        >>> modpath = basename(modname_to_modpath('_ctypes'))
-        >>> assert 'ctypes' in modpath
+        >>> modpath = modname_to_modpath('json')
+        >>> assert 'json' in modpath
     """
     if hide_main or sys_path:
         modpath = _syspath_modname_to_modpath(modname, sys_path)
@@ -740,18 +838,25 @@ def modname_to_modpath(modname, hide_init=True, hide_main=False, sys_path=None):
         #     modpath = _importlib_modname_to_modpath(modname)
         # except Exception:
         #     modpath = _syspath_modname_to_modpath(modname, sys_path)
-        # modpath = _pkgutil_modname_to_modpath(modname, sys_path)
         modpath = _syspath_modname_to_modpath(modname, sys_path)
 
     if modpath is None:
         return None
 
-    modpath = normalize_modpath(modpath, hide_init=hide_init,
-                                hide_main=hide_main)
+    modpath = normalize_modpath(
+        modpath, hide_init=hide_init, hide_main=hide_main
+    )
+
+    if typing.TYPE_CHECKING:
+        modpath = typing.cast(str, modpath)
     return modpath
 
 
-def normalize_modpath(modpath, hide_init=True, hide_main=False):
+def normalize_modpath(
+    modpath: str | os.PathLike,
+    hide_init: bool = True,
+    hide_main: bool = False,
+) -> str | os.PathLike:
     """
     Normalizes __init__ and __main__ paths.
 
@@ -807,8 +912,13 @@ def normalize_modpath(modpath, hide_init=True, hide_main=False):
     return modpath
 
 
-def modpath_to_modname(modpath, hide_init=True, hide_main=False, check=True,
-                       relativeto=None):
+def modpath_to_modname(
+    modpath: str,
+    hide_init: bool = True,
+    hide_main: bool = False,
+    check: bool = True,
+    relativeto: str | None = None,
+) -> str:
     """
     Determines importable name from file path
 
@@ -862,12 +972,11 @@ def modpath_to_modname(modpath, hide_init=True, hide_main=False, check=True,
         >>> assert modpath_to_modname(dirname(xdoctest.__file__.replace('.pyc', '.py'))) == 'xdoctest'
 
     Example:
-        >>> # xdoctest: +REQUIRES(CPython)
         >>> from ubelt.util_import import modpath_to_modname
         >>> from ubelt.util_import import modname_to_modpath
-        >>> modpath = modname_to_modpath('_ctypes')
+        >>> modpath = modname_to_modpath('json')
         >>> modname = modpath_to_modname(modpath)
-        >>> assert modname == '_ctypes'
+        >>> assert modname == 'json'
 
     Example:
         >>> from ubelt.util_import import modpath_to_modname
@@ -880,8 +989,9 @@ def modpath_to_modname(modpath, hide_init=True, hide_main=False, check=True,
             raise ValueError('modpath={} does not exist'.format(modpath))
     modpath_ = abspath(expanduser(modpath))
 
-    modpath_ = normalize_modpath(modpath_, hide_init=hide_init,
-                                 hide_main=hide_main)
+    modpath_ = normalize_modpath(
+        modpath_, hide_init=hide_init, hide_main=hide_main
+    )
     if relativeto:
         dpath = dirname(abspath(expanduser(relativeto)))
         rel_modpath = relpath(modpath_, dpath)
@@ -896,7 +1006,9 @@ def modpath_to_modname(modpath, hide_init=True, hide_main=False, check=True,
     return modname
 
 
-def split_modpath(modpath, check=True):
+def split_modpath(
+    modpath: str | os.PathLike, check: bool = True
+) -> tuple[str, str]:
     """
     Splits the modpath into the dir that must be in PYTHONPATH for the module
     to be imported and the modulepath relative to this directory.
@@ -943,7 +1055,11 @@ def split_modpath(modpath, check=True):
     return dpath, rel_modpath
 
 
-def is_modname_importable(modname, sys_path=None, exclude=None):
+def is_modname_importable(
+    modname: str,
+    sys_path: list | None = None,
+    exclude: list | None = None,
+) -> bool:
     """
     Determines if a modname is importable based on your current sys.path
 
@@ -965,8 +1081,9 @@ def is_modname_importable(modname, sys_path=None, exclude=None):
         >>> is_modname_importable('xdoctest', sys_path=[])
         False
     """
-    modpath = _syspath_modname_to_modpath(modname, sys_path=sys_path,
-                                          exclude=exclude)
+    modpath = _syspath_modname_to_modpath(
+        modname, sys_path=sys_path, exclude=exclude
+    )
     flag = bool(modpath is not None)
     return flag
 
@@ -1047,26 +1164,43 @@ def _parse_static_node_value(node):
     Extract a constant value from a node if possible
     """
     import ast
-    from collections import OrderedDict
     import numbers
-    if (isinstance(node, ast.Constant) and isinstance(node.value, numbers.Number) if IS_PY_GE_308 else isinstance(node, ast.Num)):
-        value = node.value if IS_PY_GE_308 else node.n
-    elif (isinstance(node, ast.Constant) and isinstance(node.value, str) if IS_PY_GE_308 else isinstance(node, ast.Str)):
-        value = node.value if IS_PY_GE_308 else node.s
-    elif isinstance(node, ast.List):
-        value = list(map(_parse_static_node_value, node.elts))
-    elif isinstance(node, ast.Tuple):
-        value = tuple(map(_parse_static_node_value, node.elts))
-    elif isinstance(node, (ast.Dict)):
+    from collections import OrderedDict
+
+    if IS_PY_GE_308:
+        if isinstance(node, ast.Constant) and isinstance(
+            node.value, numbers.Number
+        ):
+            return node.value
+        if isinstance(node, ast.Constant) and isinstance(node.value, str):
+            return node.value
+    else:  # nocover
+        num_type = getattr(ast, 'Num', None)
+        str_type = getattr(ast, 'Str', None)
+        if num_type is not None and isinstance(node, num_type):
+            return node.n
+        if str_type is not None and isinstance(node, str_type):
+            return node.s
+
+    if isinstance(node, ast.List):
+        return list(map(_parse_static_node_value, node.elts))
+    if isinstance(node, ast.Tuple):
+        return tuple(map(_parse_static_node_value, node.elts))
+    if isinstance(node, ast.Dict):
         keys = map(_parse_static_node_value, node.keys)
         values = map(_parse_static_node_value, node.values)
-        value = OrderedDict(zip(keys, values))
-        # value = dict(zip(keys, values))
-    elif IS_PY_LT_314 and isinstance(node, (ast.NameConstant)):  # nocover
-        value = node.value
-    elif isinstance(node, ast.Constant):
-        value = node.value
-    else:
-        raise TypeError('Cannot parse a static value from non-static node '
-                        'of type: {!r}'.format(type(node)))
-    return value
+        return OrderedDict(zip(keys, values))
+
+    if IS_PY_LT_314:  # nocover
+        nameconst_type = getattr(ast, 'NameConstant', None)
+        if nameconst_type is not None and isinstance(node, nameconst_type):
+            return node.value
+
+    if isinstance(node, ast.Constant):  # nocover
+        return node.value
+
+    raise TypeError(
+        'Cannot parse a static value from non-static node of type: {!r}'.format(
+            type(node)
+        )
+    )
