@@ -9,11 +9,20 @@ References:
     .. [PypiDictDigger] https://pypi.org/project/dict_digger/
     .. [PypiDeepDiff] https://pypi.org/project/deepdiff/
 """
-from math import isclose
+
+from __future__ import annotations
+
+import typing
 from collections.abc import Generator
-from typing import NamedTuple, Tuple, Any
+from math import isclose
+
 # from collections.abc import Iterable
 
+if typing.TYPE_CHECKING:
+    from types import TracebackType
+    from typing import Any, Mapping, MutableMapping, MutableSequence
+
+
 try:
     from functools import cache
 except ImportError:  # nocover
@@ -29,12 +38,13 @@ def _lazy_numpy():
     return np
 
 
-class Difference(NamedTuple):
+class Difference(typing.NamedTuple):
     """
     A result class of indexable_diff that organizes what the difference between
     the indexables is.
     """
-    path: Tuple
+
+    path: tuple
     value1: Any
     value2: Any
 
@@ -178,14 +188,27 @@ class IndexableWalker(Generator):
         >>> assert num_iters == 3 + 3 * 5
     """
 
-    def __init__(self, data, dict_cls=(dict,), list_cls=(list, tuple)):
+    data: dict | list | tuple
+    dict_cls: tuple[type, ...]
+    list_cls: tuple[type, ...]
+    indexable_cls: tuple[type, ...]
+    _walk_gen: None | Generator[tuple[list, Any], Any, Any]
+
+    def __init__(
+        self,
+        data,
+        dict_cls: tuple[type, ...] = (dict,),
+        list_cls: tuple[type, ...] = (list, tuple),
+    ) -> None:
         self.data = data
         self.dict_cls = dict_cls
         self.list_cls = list_cls
         self.indexable_cls = self.dict_cls + self.list_cls
         self._walk_gen = None
 
-    def __iter__(self):
+    def __iter__(
+        self,
+    ) -> Generator[tuple[list, typing.Any], typing.Any, typing.Any]:
         """
         Iterates through the indexable ``self.data``
 
@@ -200,7 +223,7 @@ class IndexableWalker(Generator):
         self._walk_gen = self._walk()
         return self._walk_gen
 
-    def __next__(self):
+    def __next__(self) -> typing.Any:
         """
         returns next item from this generator
 
@@ -213,15 +236,24 @@ class IndexableWalker(Generator):
 
     # TODO: maybe we implement a map function?
 
-    def send(self, arg):
+    def send(self, arg) -> None:
         """
         send(arg) -> send 'arg' into generator,
         return next yielded value or raise StopIteration.
         """
         # Note: this will error if called before __next__
+        if self._walk_gen is None:
+            raise TypeError(
+                'cannot send to walker before iteration has started'
+            )
         self._walk_gen.send(arg)
 
-    def throw(self, typ, val=None, tb=None):  # type: ignore
+    def throw(
+        self,
+        typ: typing.Any,
+        val: object | None = None,
+        tb: TracebackType | None = None,
+    ) -> typing.Any:
         """
         throw(typ[,val[,tb]]) -> raise exception in generator,
         return next yielded value or raise StopIteration.
@@ -246,7 +278,7 @@ class IndexableWalker(Generator):
         """
         raise StopIteration
 
-    def __setitem__(self, path, value):
+    def __setitem__(self, path: list, value: Any) -> None:
         """
         Set nested value by path
 
@@ -255,6 +287,7 @@ class IndexableWalker(Generator):
             value (Any): new value
         """
         import itertools as it
+
         d = self.data
         # note: slice unpack seems faster in 3.9 at least, dont change
         # ~/misc/tests/python/bench_unpack.py
@@ -266,9 +299,11 @@ class IndexableWalker(Generator):
         # *prefix, key = path
         for k in prefix:
             d = d[k]
+        if typing.TYPE_CHECKING:
+            d = typing.cast(MutableMapping[Any, Any] | MutableSequence[Any], d)
         d[key] = value
 
-    def __getitem__(self, path):
+    def __getitem__(self, path: list) -> Any:
         """
         Get nested value by path
 
@@ -279,6 +314,7 @@ class IndexableWalker(Generator):
             Any: value
         """
         import itertools as it
+
         d = self.data
         # Using islice allows path to be a list or deque
         key_index = len(path) - 1
@@ -290,7 +326,7 @@ class IndexableWalker(Generator):
             d = d[k]
         return d[key]
 
-    def __delitem__(self, path):
+    def __delitem__(self, path: list) -> None:
         """
         Remove nested value by path
 
@@ -309,6 +345,8 @@ class IndexableWalker(Generator):
         # *prefix, key = path
         for k in prefix:
             d = d[k]
+        if typing.TYPE_CHECKING:
+            d = typing.cast(MutableMapping[Any, Any] | MutableSequence[Any], d)
         del d[key]
 
     def keys(self, non_leaf=False):
@@ -401,6 +439,8 @@ class IndexableWalker(Generator):
             if isinstance(_data, self.list_cls):
                 items = enumerate(_data)
             elif isinstance(_data, self.dict_cls):
+                if typing.TYPE_CHECKING:
+                    _data = typing.cast(Mapping[Any, Any], _data)
                 items = _data.items()
             else:
                 raise TypeError(type(_data))
@@ -421,7 +461,14 @@ class IndexableWalker(Generator):
                     if isinstance(value, self.indexable_cls):
                         stack.append((value, path))
 
-    def allclose(self, other, rel_tol=1e-9, abs_tol=0.0, equal_nan=False, return_info=False):
+    def allclose(
+        self,
+        other: IndexableWalker | list | dict,
+        rel_tol: float = 1e-9,
+        abs_tol: float = 0.0,
+        equal_nan: bool = False,
+        return_info: bool = False,
+    ) -> bool | tuple[bool, dict]:
         """
         Walks through this and another nested data structures and checks if
         everything is roughly the same.
@@ -534,25 +581,28 @@ class IndexableWalker(Generator):
         if isinstance(other, IndexableWalker):
             walker2 = other
         else:
-            walker2 = IndexableWalker(other, dict_cls=self.dict_cls,
-                                      list_cls=self.list_cls)
+            walker2 = IndexableWalker(
+                other, dict_cls=self.dict_cls, list_cls=self.list_cls
+            )
 
         _isclose_fn, _iskw = _make_isclose_fn(rel_tol, abs_tol, equal_nan)
 
         flat_items1 = [
-            (path, value) for path, value in walker1
-            if not isinstance(value, walker1.indexable_cls) or len(value) == 0]
+            (path, value)
+            for path, value in walker1
+            if not isinstance(value, walker1.indexable_cls) or len(value) == 0
+        ]
         flat_items2 = [
-            (path, value) for path, value in walker2
-            if not isinstance(value, walker1.indexable_cls) or len(value) == 0]
+            (path, value)
+            for path, value in walker2
+            if not isinstance(value, walker1.indexable_cls) or len(value) == 0
+        ]
 
         flat_items1 = sorted(flat_items1)
         flat_items2 = sorted(flat_items2)
 
         if len(flat_items1) != len(flat_items2):
-            info = {
-                'faillist': ['length mismatch']
-            }
+            info = {'faillist': ['length mismatch']}
             final_flag = False
         else:
             passlist = []
@@ -566,8 +616,9 @@ class IndexableWalker(Generator):
                 # TODO: Could add a numpy optimization here.
 
                 flag = (v1 == v2) or (
-                    isinstance(v1, float) and isinstance(v2, float) and
-                    _isclose_fn(v1, v2, **_iskw)
+                    isinstance(v1, float)
+                    and isinstance(v2, float)
+                    and _isclose_fn(v1, v2, **_iskw)
                 )
                 if flag:
                     passlist.append(p1)
@@ -581,10 +632,12 @@ class IndexableWalker(Generator):
             }
 
         if return_info:
-            info.update({
-                'walker1': walker1,
-                'walker2': walker2,
-            })
+            info.update(
+                {
+                    'walker1': walker1,
+                    'walker2': walker2,
+                }
+            )
             return final_flag, info
         else:
             return final_flag
@@ -667,15 +720,20 @@ class IndexableWalker(Generator):
         if isinstance(other, IndexableWalker):
             walker2 = other
         else:
-            walker2 = IndexableWalker(other, dict_cls=self.dict_cls,
-                                      list_cls=self.list_cls)
+            walker2 = IndexableWalker(
+                other, dict_cls=self.dict_cls, list_cls=self.list_cls
+            )
         # TODO: numpy optimizations
         flat_items1 = {
-            tuple(path): value for path, value in walker1
-            if not isinstance(value, walker1.indexable_cls) or len(value) == 0}
+            tuple(path): value
+            for path, value in walker1
+            if not isinstance(value, walker1.indexable_cls) or len(value) == 0
+        }
         flat_items2 = {
-            tuple(path): value for path, value in walker2
-            if not isinstance(value, walker1.indexable_cls) or len(value) == 0}
+            tuple(path): value
+            for path, value in walker2
+            if not isinstance(value, walker1.indexable_cls) or len(value) == 0
+        }
 
         common = flat_items1.keys() & flat_items2.keys()
         unique1 = flat_items1.keys() - flat_items2.keys()
@@ -690,11 +748,12 @@ class IndexableWalker(Generator):
         for key in common:
             v1 = flat_items1[key]
             v2 = flat_items2[key]
-            flag = (v1 == v2)
+            flag = v1 == v2
             if not flag:
                 flag = (
-                    isinstance(v1, float) and isinstance(v2, float) and
-                    _isclose_fn(v1, v2, **_iskw)
+                    isinstance(v1, float)
+                    and isinstance(v2, float)
+                    and _isclose_fn(v1, v2, **_iskw)
                 )
                 num_approximations += flag
             if flag:
@@ -735,7 +794,13 @@ def _make_isclose_fn(rel_tol, abs_tol, equal_nan):
     return _isclose_fn, _iskw
 
 
-def indexable_allclose(items1, items2, rel_tol=1e-9, abs_tol=0.0, return_info=False):
+def indexable_allclose(
+    items1: dict | list | tuple,
+    items2: dict | list | tuple,
+    rel_tol: float = 1e-9,
+    abs_tol: float = 0.0,
+    return_info: bool = False,
+) -> bool | tuple[bool, dict]:
     """
     Walks through two nested data structures and ensures that everything is
     roughly the same.
@@ -784,15 +849,18 @@ def indexable_allclose(items1, items2, rel_tol=1e-9, abs_tol=0.0, return_info=Fa
         >>> print('flag = {!r}'.format(flag))
     """
     from ubelt.util_deprecate import schedule_deprecation
+
     schedule_deprecation(
-        'ubelt', 'indexable_allclose', 'function',
-        migration=(
-            'Use `ub.IndexableWalker(items1).allclose(items2)` instead'
-        ))
+        'ubelt',
+        'indexable_allclose',
+        'function',
+        migration=('Use `ub.IndexableWalker(items1).allclose(items2)` instead'),
+    )
     walker1 = IndexableWalker(items1)
     walker2 = IndexableWalker(items2)
-    return walker1.allclose(walker2, rel_tol=rel_tol, abs_tol=abs_tol,
-                            return_info=return_info)
+    return walker1.allclose(
+        walker2, rel_tol=rel_tol, abs_tol=abs_tol, return_info=return_info
+    )
 
 
 # Nested = IndexableWalker
