@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 """
 Defines the function :func:`urepr`, which allows for a bit more customization
 than :func:`repr` or :func:`pprint.pformat`. See the docstring for more details.
@@ -50,14 +52,22 @@ As of ubelt 1.1.0 you can now access and update the default extensions via the
 >>>     return "monkey2({})".format(data)
 >>> print(ub.urepr({1: float('nan'), 2: float('inf'), 3: 3.0}, nl=0))
 """
+
 import collections
-from ubelt import util_str
-from ubelt import util_list
+import typing
+
+from ubelt import util_list, util_str
+
+if typing.TYPE_CHECKING:
+    from typing import Any, Callable
 
 __all__ = ['urepr', 'ReprExtensions']
 
+# TODO: add a wrapper function so _return_info is not available in the top
+# level urepr. This will make type annotations more predictable.
 
-def urepr(data, **kwargs):
+
+def urepr(data: object, **kwargs: Any) -> str | tuple[str, dict[str, int]]:
     """
     Makes a pretty string representation of ``data``.
 
@@ -143,7 +153,7 @@ def urepr(data, **kwargs):
             which are currently not configurable. This may be modified in the
             future. Defaults to ' '.
 
-        sort (bool | callable | None):
+        sort (bool | collections.abc.Callable[..., object] | None):
             if 'auto', then sort unordered collections, but keep the ordering
             of ordered collections. This option attempts to be deterministic in
             most cases. Defaults to None.
@@ -271,8 +281,8 @@ def urepr(data, **kwargs):
         kwargs['itemsep'] = kwargs.get('itemsep', '')
         kwargs['kvsep'] = kwargs.get('kvsep', '=')
 
-    outstr = None
-    _leaf_info = None
+    outstr: str | None = None
+    _leaf_info: dict[str, int] | None = None
 
     if custom_extensions:
         func = custom_extensions.lookup(data)
@@ -293,6 +303,8 @@ def urepr(data, **kwargs):
         else:
             outstr = _format_object(data, **kwargs)
 
+    assert outstr is not None
+
     if _return_info:
         _leaf_info = _rectify_leaf_info(_leaf_info)
         return outstr, _leaf_info
@@ -300,7 +312,7 @@ def urepr(data, **kwargs):
         return outstr
 
 
-def _rectify_root_info(_root_info):
+def _rectify_root_info(_root_info: dict[str, int] | None) -> dict[str, int]:
     if _root_info is None:
         _root_info = {
             'depth': 0,
@@ -308,7 +320,7 @@ def _rectify_root_info(_root_info):
     return _root_info
 
 
-def _rectify_leaf_info(_leaf_info):
+def _rectify_leaf_info(_leaf_info: dict[str, int] | None) -> dict[str, int]:
     if _leaf_info is None:
         _leaf_info = {
             'max_height': 0,
@@ -355,6 +367,7 @@ class ReprExtensions:
             'b': I can do anything here
         }
     """
+
     # set_types = [set, frozenset]
     # list_types = [list, tuple]
     # dict_types = [dict]
@@ -366,37 +379,42 @@ class ReprExtensions:
     # def sequence_types(cls):
     #     return cls.list_types + cls.set_types
 
-    def __init__(self):
-        self._type_registry = {}      # type: Dict[Type, Callable]  # NOQA
-        self._typename_registry = {}  # type: Dict[str, Callable]  # NOQA
-        self._lazy_queue = []         # type: List[Callable]  # NOQA
+    def __init__(self) -> None:
+        self._type_registry: dict[type, Callable[..., Any]] = {}
+        self._typename_registry: dict[str, Callable[..., Any]] = {}
+        self._lazy_queue: list[Callable[[], None]] = []
         # self._lazy_registrations = [
         #     self._register_numpy_extensions,
         #     self._register_builtin_extensions,
         # ]
 
-    def register(self, key):
+    def register(
+        self,
+        key: type | tuple[type, ...] | str,
+    ) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
         """
         Registers a custom formatting function with ub.urepr
 
         Args:
-            key (Type | Tuple[Type] | str): indicator of the type
+            key (type | tuple[type, ...] | str): indicator of the type
 
         Returns:
-            Callable: decorator function
+            collections.abc.Callable: decorator function
         """
-        def _decorator(func):
+
+        def _decorator(func: Callable[..., Any]) -> Callable[..., Any]:
             if isinstance(key, tuple):
                 for t in key:
                     self._type_registry[t] = func
-            if isinstance(key, str):
+            elif isinstance(key, str):
                 self._typename_registry[key] = func
             else:
                 self._type_registry[key] = func
             return func
+
         return _decorator
 
-    def lookup(self, data):
+    def lookup(self, data: Any) -> Callable[..., Any] | None:
         """
         Returns an appropriate function to format ``data`` if one has been
         registered.
@@ -405,7 +423,7 @@ class ReprExtensions:
             data (Any): an instance that may have a registered formatter
 
         Returns:
-            Callable: the formatter for the given type
+            collections.abc.Callable[..., object] | None: the formatter for the given type
         """
         # Evaluate the lazy queue if anything is in it
         if self._lazy_queue:
@@ -423,7 +441,7 @@ class ReprExtensions:
         func = self._typename_registry.get(typename, None)
         return func
 
-    def _register_pandas_extensions(self):
+    def _register_pandas_extensions(self) -> None:
         """
         Example:
             >>> # xdoctest: +REQUIRES(module:pandas)
@@ -437,11 +455,13 @@ class ReprExtensions:
             >>> print(ub.urepr(data, precision=2))
             >>> print(ub.urepr({'akeyfdfj': data}, precision=2))
         """
+
         @self.register('DataFrame')
-        def format_pandas(data, **kwargs):  # nocover
+        def format_pandas(data: Any, **kwargs: Any) -> str:  # nocover
             precision = kwargs.get('precision', None)
-            float_format = (None if precision is None
-                            else '%.{}f'.format(precision))
+            float_format = (
+                None if precision is None else '%.{}f'.format(precision)
+            )
             formatted = data.to_string(float_format=float_format)
             return formatted
 
@@ -473,7 +493,7 @@ class ReprExtensions:
     #         # formatted = ub.hzcat('Tensor(' + formatted + ')')
     #         return formatted
 
-    def _register_numpy_extensions(self):
+    def _register_numpy_extensions(self) -> None:
         """
         Example:
             >>> # xdoctest: +REQUIRES(module:numpy)
@@ -506,15 +526,19 @@ class ReprExtensions:
 
         # TODO: should we register numpy using the new string method?
         import numpy as np
+
         @self.register(np.ndarray)
-        def format_ndarray(data, **kwargs):
+        def format_ndarray(data: Any, **kwargs: Any) -> str:
             import re
+
             strvals = kwargs.get('sv', kwargs.get('strvals', False))
             itemsep = kwargs.get('itemsep', ' ')
             precision = kwargs.get('precision', None)
             suppress_small = kwargs.get('supress_small', None)
             max_line_width = kwargs.get('max_line_width', None)
-            with_dtype = kwargs.get('with_dtype', kwargs.get('dtype', not strvals))
+            with_dtype = kwargs.get(
+                'with_dtype', kwargs.get('dtype', not strvals)
+            )
             newlines = kwargs.pop('nl', kwargs.pop('newlines', 1))
 
             # if with_dtype and strvals:
@@ -541,7 +565,9 @@ class ReprExtensions:
                 if with_dtype:
                     dtype_repr = data.dtype.name
                     # dtype_repr = np.core.arrayprint.dtype_short_repr(data.dtype)
-                    suffix = ',{}dtype={}.{})'.format(itemsep, np_nice, dtype_repr)
+                    suffix = ',{}dtype={}.{})'.format(
+                        itemsep, np_nice, dtype_repr
+                    )
                 else:
                     suffix = ')'
 
@@ -550,11 +576,14 @@ class ReprExtensions:
                 prefix = modname + '.empty('
                 body = repr(tuple(map(int, data.shape)))
             else:
-                body = np.array2string(data, precision=precision,
-                                       separator=separator,
-                                       suppress_small=suppress_small,
-                                       prefix=prefix,
-                                       max_line_width=max_line_width)
+                body = np.array2string(
+                    data,
+                    precision=precision,
+                    separator=separator,
+                    suppress_small=suppress_small,
+                    prefix=prefix,
+                    max_line_width=max_line_width,
+                )
 
             if not strvals:
                 # Handle special float values inf / nan
@@ -570,9 +599,9 @@ class ReprExtensions:
         # Hack, make sure we also register numpy floats
         self.register(np.float32)(self._type_registry[float])
 
-    def _register_builtin_extensions(self):
+    def _register_builtin_extensions(self) -> None:
         @self.register(float)
-        def format_float(data, **kwargs):
+        def format_float(data: float, **kwargs: Any) -> str:
             precision = kwargs.get('precision', None)
             strvals = kwargs.get('sv', kwargs.get('strvals', False))
 
@@ -592,23 +621,25 @@ class ReprExtensions:
                 # json.loads("[Infinity, NaN]")
                 # json.dumps([float('inf'), float('nan')])
                 import math
+
                 if math.isinf(data) or math.isnan(data):
                     text = "float('{}')".format(text)
 
             return text
 
         @self.register(slice)
-        def format_slice(data, **kwargs):
+        def format_slice(data: slice, **kwargs: Any) -> str:
             if kwargs.get('itemsep', ' ') == '':
                 return 'slice(%r,%r,%r)' % (data.start, data.stop, data.step)
             else:
                 return _format_object(data, **kwargs)
 
+
 _REPR_EXTENSIONS = ReprExtensions()
 _REPR_EXTENSIONS._register_builtin_extensions()
 
 
-def _lazy_init():
+def _lazy_init() -> None:
     """
     Only called in the case where we encounter an unknown type that a commonly
     used external library might have. For now this is just numpy. Numpy is
@@ -623,10 +654,11 @@ def _lazy_init():
     except ImportError:  # nocover
         pass
 
+
 _REPR_EXTENSIONS._lazy_queue.append(_lazy_init)
 
 
-def _format_object(val, **kwargs):
+def _format_object(val: Any, **kwargs: Any) -> str:
     stritems = kwargs.get('si', kwargs.get('stritems', False))
     strvals = stritems or kwargs.get('sv', kwargs.get('strvals', False))
     base_valfunc = str if strvals else repr
@@ -634,7 +666,7 @@ def _format_object(val, **kwargs):
     return itemstr
 
 
-def _format_list(list_, **kwargs):
+def _format_list(list_: Any, **kwargs: Any) -> tuple[str, dict[str, int]]:
     """
     Makes a pretty printable / human-readable string representation of a
     sequence. In most cases this string could be evaled.
@@ -646,7 +678,7 @@ def _format_list(list_, **kwargs):
             stritems, strkeys, explicit, sort, key_order, maxlen
 
     Returns:
-        Tuple[str, Dict] : retstr, _leaf_info
+        tuple[str, dict[str, object]]: retstr, _leaf_info
 
     Example:
         >>> print(_format_list([])[0])
@@ -677,18 +709,26 @@ def _format_list(list_, **kwargs):
         nobraces = False  # force braces to prevent empty output
 
     is_tuple = isinstance(list_, tuple)
-    is_set = isinstance(list_, (set, frozenset,))
+    is_set = isinstance(
+        list_,
+        (
+            set,
+            frozenset,
+        ),
+    )
     if nobraces:
         lbr, rbr = '', ''
     elif is_tuple:
-        lbr, rbr  = '(', ')'
+        lbr, rbr = '(', ')'
     elif is_set:
-        lbr, rbr  = '{', '}'
+        lbr, rbr = '{', '}'
     else:
-        lbr, rbr  = '[', ']'
+        lbr, rbr = '[', ']'
 
     # Doesn't actually put in trailing comma if on same line
-    trailing_sep = kwargs.get('trailsep', kwargs.get('trailing_sep', newlines > 0 and len(itemstrs)))
+    trailing_sep = kwargs.get(
+        'trailsep', kwargs.get('trailing_sep', newlines > 0 and len(itemstrs))
+    )
 
     # The trailing separator is always needed for single item tuples
     if is_tuple and len(list_) <= 1:
@@ -697,12 +737,23 @@ def _format_list(list_, **kwargs):
     if len(itemstrs) == 0:
         newlines = False
 
-    retstr = _join_itemstrs(itemstrs, itemsep, newlines, _leaf_info, nobraces,
-                            trailing_sep, compact_brace, lbr, rbr)
+    retstr = _join_itemstrs(
+        itemstrs,
+        itemsep,
+        newlines,
+        _leaf_info,
+        nobraces,
+        trailing_sep,
+        compact_brace,
+        lbr,
+        rbr,
+    )
     return retstr, _leaf_info
 
 
-def _format_dict(dict_, **kwargs):
+def _format_dict(
+    dict_: dict[Any, Any], **kwargs: Any
+) -> tuple[str, dict[str, int] | None]:
     """
     Makes a pretty printable / human-readable string representation of a
     dictionary. In most cases this string could be evaled.
@@ -731,7 +782,7 @@ def _format_dict(dict_, **kwargs):
         nobr (bool): removes outer braces. Defaults to False.
 
     Returns:
-        Tuple[str, Dict] : retstr, _leaf_info
+        tuple[str, dict[str, object]]: retstr, _leaf_info
 
     Example:
         >>> from ubelt.util_repr import *  # NOQA
@@ -774,17 +825,24 @@ def _format_dict(dict_, **kwargs):
     # kwargs['cbr'] = _rectify_countdown_or_bool(compact_brace)
 
     # Doesn't actually put in trailing comma if on same line
-    trailing_sep = kwargs.get('trailsep', kwargs.get('trailing_sep', newlines > 0))
+    trailing_sep = kwargs.get(
+        'trailsep', kwargs.get('trailing_sep', newlines > 0)
+    )
     explicit = kwargs.get('explicit', False)
     itemsep = kwargs.get('itemsep', ' ')
 
     align = kwargs.get('align', False)
+    align_char: str | None
     if align and not isinstance(align, str):
         default_kvsep = ': '
         if explicit:
             default_kvsep = '='
         kvsep = kwargs.get('kvsep', default_kvsep)
-        align = kvsep
+        align_char = kvsep
+    elif align:
+        align_char = align
+    else:
+        align_char = None
 
     if len(dict_) == 0:
         retstr = 'dict()' if explicit else '{}'
@@ -797,13 +855,33 @@ def _format_dict(dict_, **kwargs):
             lbr, rbr = 'dict(', ')'
         else:
             lbr, rbr = '{', '}'
-        retstr = _join_itemstrs(itemstrs, itemsep, newlines, _leaf_info, nobraces,
-                                trailing_sep, compact_brace, lbr, rbr, align)
+        retstr = _join_itemstrs(
+            itemstrs,
+            itemsep,
+            newlines,
+            _leaf_info,
+            nobraces,
+            trailing_sep,
+            compact_brace,
+            lbr,
+            rbr,
+            align_char,
+        )
     return retstr, _leaf_info
 
 
-def _join_itemstrs(itemstrs, itemsep, newlines, _leaf_info, nobraces,
-                   trailing_sep, compact_brace, lbr, rbr, align=False):
+def _join_itemstrs(
+    itemstrs: list[str],
+    itemsep: str,
+    newlines: int | bool,
+    _leaf_info: dict[str, int],
+    nobraces: bool,
+    trailing_sep: bool,
+    compact_brace: bool,
+    lbr: str,
+    rbr: str,
+    align: str | None = None,
+) -> str:
     """
     Joins string-ified items with separators newlines and container-braces.
     """
@@ -819,7 +897,7 @@ def _join_itemstrs(itemstrs, itemsep, newlines, _leaf_info, nobraces,
     if use_newline:
         sep = ',\n'
         if nobraces:
-            if align:
+            if align is not None:
                 itemstrs = _align_lines(itemstrs, character=align)
             body_str = sep.join(itemstrs)
             if trailing_sep and len(itemstrs) > 0:
@@ -836,7 +914,7 @@ def _join_itemstrs(itemstrs, itemsep, newlines, _leaf_info, nobraces,
                 prefix = ' ' * 4
                 indented = [util_str.indent(s, prefix) for s in itemstrs]
 
-            if align:
+            if align is not None:
                 indented = _align_lines(indented, character=align)
 
             body_str = sep.join(indented)
@@ -844,20 +922,22 @@ def _join_itemstrs(itemstrs, itemsep, newlines, _leaf_info, nobraces,
                 body_str += ','
             if compact_brace:
                 # Why can we modify the indentation here but not above?
-                braced_body_str = (lbr + body_str.replace('\n', '\n ') + rbr)
+                braced_body_str = lbr + body_str.replace('\n', '\n ') + rbr
             else:
-                braced_body_str = (lbr + '\n' + body_str + '\n' + rbr)
+                braced_body_str = lbr + '\n' + body_str + '\n' + rbr
             retstr = braced_body_str
     else:
         sep = ',' + itemsep
         body_str = sep.join(itemstrs)
         if trailing_sep and len(itemstrs) > 0:
             body_str += ','
-        retstr  = (lbr + body_str +  rbr)
+        retstr = lbr + body_str + rbr
     return retstr
 
 
-def _dict_itemstrs(dict_, **kwargs):
+def _dict_itemstrs(
+    dict_: dict[Any, Any], **kwargs: Any
+) -> tuple[list[str], dict[str, int]]:
     """
     Create a string representation for each item in a dict.
 
@@ -875,6 +955,7 @@ def _dict_itemstrs(dict_, **kwargs):
         >>> assert char_order == ['b', 'g', 'l', 'm', 's', 'w']
     """
     import ubelt as ub
+
     explicit = kwargs.get('explicit', False)
     kwargs['explicit'] = _rectify_countdown_or_bool(explicit)
     precision = kwargs.get('precision', None)
@@ -886,7 +967,7 @@ def _dict_itemstrs(dict_, **kwargs):
         default_kvsep = '='
     kvsep = kwargs.get('kvsep', default_kvsep)
 
-    def make_item_str(key, val):
+    def make_item_str(key: Any, val: Any) -> tuple[str, dict[str, int]]:
         if explicit or kwargs.get('strkeys', default_strkeys):
             key_str = str(key)
         else:
@@ -894,7 +975,8 @@ def _dict_itemstrs(dict_, **kwargs):
 
         prefix = key_str + kvsep
         kwargs['_return_info'] = True
-        val_str, _leaf_info = urepr(val, **kwargs)
+        _ret = urepr(val, **kwargs)
+        val_str, _leaf_info = _ret
 
         # If the first line does not end with an open nest char
         # (e.g. for ndarrays), otherwise we need to worry about
@@ -915,7 +997,7 @@ def _dict_itemstrs(dict_, **kwargs):
                 item_str = ub.hzcat([prefix, val_str])
         else:
             item_str = prefix + val_str
-        return item_str, _leaf_info
+        return item_str, _leaf_info  # type: ignore[invalid-return-type]
 
     items = list(dict_.items())
     _tups = [make_item_str(key, val) for (key, val) in items]
@@ -954,17 +1036,20 @@ def _dict_itemstrs(dict_, **kwargs):
     return itemstrs, _leaf_info
 
 
-def _list_itemstrs(list_, **kwargs):
+def _list_itemstrs(
+    list_: Any, **kwargs: Any
+) -> tuple[list[str], dict[str, int]]:
     """
     Create a string representation for each item in a list.
 
     Args:
-        list_ (Sequence):
+        list_ (collections.abc.Sequence[object]):
         **kwargs: _return_info, sort
     """
     items = list(list_)
     kwargs['_return_info'] = True
-    _tups = [urepr(item, **kwargs) for item in items]
+    _tups: list[tuple[str, dict[str, int]]]
+    _tups = [urepr(item, **kwargs) for item in items]  # type: ignore[invalid-assignment]
     itemstrs = [t[0] for t in _tups]
     max_height = max([t[1]['max_height'] for t in _tups]) if _tups else 0
     _leaf_info = {
@@ -985,7 +1070,11 @@ def _list_itemstrs(list_, **kwargs):
     return itemstrs, _leaf_info
 
 
-def _sort_itemstrs(items, itemstrs, key=None):
+def _sort_itemstrs(
+    items: list[Any],
+    itemstrs: list[str],
+    key: Callable[[Any], Any] | None = None,
+) -> list[str]:
     """
     Equivalent to ``sorted(items)`` except if ``items`` are unorderable, then
     string values are used to define an ordering.
@@ -1003,7 +1092,7 @@ def _sort_itemstrs(items, itemstrs, key=None):
     return itemstrs
 
 
-def _rectify_countdown_or_bool(count_or_bool):
+def _rectify_countdown_or_bool(count_or_bool: Any) -> bool | int:
     """
     used by recursive functions to specify which level to turn a bool on in
     counting down yields True, True, ..., False
@@ -1046,7 +1135,9 @@ def _rectify_countdown_or_bool(count_or_bool):
     return count_or_bool_
 
 
-def _align_text(text, character='=', replchar=None, pos=0):
+def _align_text(
+    text: str, character: str = '=', replchar: str | None = None, pos: int = 0
+) -> str:
     r"""
     Left justifies text on the left side of character
 
@@ -1074,7 +1165,12 @@ def _align_text(text, character='=', replchar=None, pos=0):
     return new_text
 
 
-def _align_lines(line_list, character='=', replchar=None, pos=0):
+def _align_lines(
+    line_list: list[str],
+    character: str = '=',
+    replchar: str | None = None,
+    pos: int | list[int] | None = 0,
+) -> list[str]:
     r"""
     Left justifies text on the left side of character
 
@@ -1153,8 +1249,9 @@ def _align_lines(line_list, character='=', replchar=None, pos=0):
         # recursive calls
         new_lines = line_list
         for pos in pos_list:
-            new_lines = _align_lines(new_lines, character=character,
-                                     replchar=replchar, pos=pos)
+            new_lines = _align_lines(
+                new_lines, character=character, replchar=replchar, pos=pos
+            )
         return new_lines
 
     # base case
@@ -1199,5 +1296,5 @@ def _align_lines(line_list, character='=', replchar=None, pos=0):
 
 # Give the urepr function itself a reference to the default extensions
 # register method so the user can modify them without accessing this module
-urepr.extensions = _REPR_EXTENSIONS
-urepr.register = _REPR_EXTENSIONS.register
+urepr.extensions = _REPR_EXTENSIONS  # type: ignore[attr-defined]
+urepr.register = _REPR_EXTENSIONS.register  # type: ignore[attr-defined]
